@@ -1,6 +1,6 @@
 //
 //  Little cms
-//  Copyright (C) 1998-2003 Marti Maria
+//  Copyright (C) 1998-2006 Marti Maria
 //
 // Permission is hereby granted, free of charge, to any person obtaining 
 // a copy of this software and associated documentation files (the "Software"), 
@@ -48,31 +48,36 @@ int    cdecl xgetopt(int argc, char *argv[], char *optionS);
 
 // ------------------------------------------------------------------------
 
-#ifdef NON_WINDOWS
-#  ifndef stricmp
-#     define stricmp strcasecmp
-#  endif
-#endif
+extern cmsHPROFILE OpenStockProfile(const char* File);
+
 
 // Flags
 
-static BOOL Verbose                = FALSE;
-static BOOL BlackWhiteCompensation = FALSE;
-static BOOL IgnoreEmbedded         = FALSE;
-static BOOL EmbedProfile           = FALSE;
-static BOOL Width16                = FALSE;
-static BOOL GamutCheck             = FALSE;
-static BOOL lIsDeviceLink          = FALSE;
+static LCMSBOOL Verbose                = FALSE;
+static LCMSBOOL BlackWhiteCompensation = FALSE;
+static LCMSBOOL IgnoreEmbedded         = FALSE;
+static LCMSBOOL EmbedProfile           = FALSE;
+static LCMSBOOL Width16                = FALSE;
+static LCMSBOOL GamutCheck             = FALSE;
+static LCMSBOOL lIsDeviceLink          = FALSE;
+static LCMSBOOL StoreAsAlpha           = FALSE;
+static int PreserveBlack		   = 0;
+static LCMSBOOL InputLabUsingICC	   = FALSE;
+
 static int Intent                  = INTENT_PERCEPTUAL;
 static int ProofingIntent          = INTENT_PERCEPTUAL;
 static int PrecalcMode             = 1;
 static double InkLimit             = 400;
 
-static char *cInpProf  = NULL;
-static char *cOutProf  = NULL;
-static char *cProofing = NULL;
+static double ObserverAdaptationState = 0;
 
 
+static const char *cInpProf  = NULL;
+static const char *cOutProf  = NULL;
+static const char *cProofing = NULL;
+
+static const char* SaveEmbedded = NULL;
+
 // Console error & warning
 
 
@@ -123,6 +128,25 @@ void FatalError(const char *frm, ...)
        exit(1);
 }
 
+// Issue a warning
+
+static
+void Warning(const char *frm, ...)
+{
+       va_list args;
+
+       va_start(args, frm);
+       ConsoleWarningHandler("TIFFICC", frm, args);
+       va_end(args);
+}
+
+
+static
+int MyErrorHandler(int ErrorCode, const char *ErrorText)
+{
+    FatalError("%s", ErrorText);
+    return 0;
+}
 
 // Out of mem
 
@@ -172,7 +196,16 @@ DWORD GetInputPixelType(TIFF *Bank)
      // Any alpha?
 
      TIFFGetFieldDefaulted(Bank, TIFFTAG_EXTRASAMPLES, &extra, &info);
-     ColorChannels = spp - extra;
+
+     // Read alpha channels as colorant
+
+     if (StoreAsAlpha) {
+
+         ColorChannels = spp;
+         extra = 0;
+     }
+     else
+        ColorChannels = spp - extra;
 
      switch (Photometric) {
 
@@ -205,8 +238,32 @@ DWORD GetInputPixelType(TIFF *Bank)
            if (ColorChannels == 6)
                   pt = PT_HiFi;
            else
+           if (ColorChannels == 7)
+                  pt = PT_HiFi7;
+           else
            if (ColorChannels == 8)
                   pt = PT_HiFi8;           
+           else
+           if (ColorChannels == 9)
+                  pt = PT_HiFi9;           
+           else
+           if (ColorChannels == 10)
+                  pt = PT_HiFi10;           
+           else
+           if (ColorChannels == 11)
+                  pt = PT_HiFi11;           
+           else
+		   if (ColorChannels == 12)
+                  pt = PT_HiFi8;           
+		   else
+           if (ColorChannels == 13)
+                  pt = PT_HiFi13;           
+           else
+           if (ColorChannels == 14)
+                  pt = PT_HiFi14;           
+           else
+           if (ColorChannels == 15)
+                  pt = PT_HiFi15;           
            else
                   FatalError("What a weird separation of %d channels?!?!", ColorChannels);
            break;
@@ -225,13 +282,16 @@ DWORD GetInputPixelType(TIFF *Bank)
            break;
 
      case 9:
+			pt = PT_Lab;
+			InputLabUsingICC = TRUE;
+			break;
+
      case PHOTOMETRIC_CIELAB:
            pt = PT_Lab;
+		   InputLabUsingICC = FALSE;
            break;
 
-     case PHOTOMETRIC_LOGL:     /* CIE Log2(L) */
-           FatalError("Hummm... I have been unable of find any of  these, please contact me at marti@littlecms.com, thanx.");
-
+    
      case PHOTOMETRIC_LOGLUV:      /* CIE Log2(L) (u',v') */
 
            TIFFSetField(Bank, TIFFTAG_SGILOGDATAFMT, SGILOGDATAFMT_16BIT);
@@ -257,71 +317,74 @@ DWORD GetInputPixelType(TIFF *Bank)
 static
 DWORD ComputeOutputFormatDescriptor(DWORD dwInput, int OutColorSpace, int bps)
 {
-   int IsPlanar  = T_PLANAR(dwInput);
-   int Channels = 0;
-
-   switch (OutColorSpace) {
-
-   case PT_GRAY:
-               Channels = 1;
-               break;
-   case PT_RGB:
-   case PT_CMY:
-   case PT_Lab:
-   case PT_YUV:
-   case PT_YCbCr:
-               Channels = 3;
-               break;
-
-   case PT_CMYK:
-               Channels = 4;
-               break;
-
-   case  PT_HiFi:
-               Channels = 6;
-               break;
-
-   case  PT_HiFi8:
-               Channels = 8;
-               break;
-   default:
-               FatalError("Unsupported output color space");
-   }
-
+    int IsPlanar  = T_PLANAR(dwInput);
+    int Channels = 0;
+    
+    switch (OutColorSpace) {
+        
+    case PT_GRAY:
+        Channels = 1;
+        break;
+    case PT_RGB:
+    case PT_CMY:
+    case PT_Lab:
+    case PT_YUV:
+    case PT_YCbCr:
+        Channels = 3;
+        break;
+        
+    case PT_CMYK:
+        Channels = 4;
+        break;
+        
+    case  PT_HiFi:
+        Channels = 6;
+        break;
+        
+    case PT_HiFi7:
+        Channels = 7;
+        break;
+        
+    case  PT_HiFi8:
+        Channels = 8;
+        break;
+        
+    case  PT_HiFi9:
+        Channels = 9;
+        break;
+        
+    case  PT_HiFi10:
+        Channels = 10;
+        break;
+        
+    case  PT_HiFi11:
+        Channels = 11;
+        break;
+        
+    case PT_HiFi12:
+        Channels = 12;
+        break;
+        
+    case  PT_HiFi13:
+        Channels = 13;
+        break;
+        
+    case  PT_HiFi14:
+        Channels = 14;
+        break;
+        
+    case  PT_HiFi15:
+        Channels = 15;
+        break;
+        
+    default:
+        FatalError("Unsupported output color space");
+    }
+    
     return (COLORSPACE_SH(OutColorSpace)|PLANAR_SH(IsPlanar)|CHANNELS_SH(Channels)|BYTES_SH(bps));
 }
 
 
-// Equivalence between ICC color spaces and lcms color spaces
-
-
-
-static
-int ICC2LCMS(icColorSpaceSignature ProfileSpace)
-{
-    
-       switch (ProfileSpace) {
-
-       case icSigGrayData: return  PT_GRAY;
-       case icSigRgbData:  return  PT_RGB;
-       case icSigCmyData:  return  PT_CMY;
-       case icSigCmykData: return  PT_CMYK;
-       case icSigYCbCrData:return  PT_YCbCr;
-       case icSigLuvData:  return  PT_YUV;
-       case icSigXYZData:  return  PT_XYZ;
-       case icSigLabData:  return  PT_Lab;
-       case icSigLuvKData: return  PT_YUVK;
-       case icSigHsvData:  return  PT_HSV;
-       case icSigHlsData:  return  PT_HLS;
-       case icSigYxyData:  return  PT_Yxy;
-
-       case icSigHexachromeData: return PT_HiFi;
-       case icSig8colorData:     return PT_HiFi8;
-
-       default:  return icMaxEnumData;
-       }
-}
-
 
 // Tile based transforms
 
@@ -449,8 +512,21 @@ static
 void WriteOutputTags(TIFF *out, int Colorspace, int BytesPerSample)
 {
     int BitsPerSample = (8 * BytesPerSample);
+    uint16 Extra[] = { EXTRASAMPLE_UNASSALPHA, 
+                       EXTRASAMPLE_UNASSALPHA, 
+                       EXTRASAMPLE_UNASSALPHA,
+                       EXTRASAMPLE_UNASSALPHA,
+					   EXTRASAMPLE_UNASSALPHA, 
+                       EXTRASAMPLE_UNASSALPHA, 
+                       EXTRASAMPLE_UNASSALPHA,
+                       EXTRASAMPLE_UNASSALPHA,
+                       EXTRASAMPLE_UNASSALPHA,
+                       EXTRASAMPLE_UNASSALPHA,
+                       EXTRASAMPLE_UNASSALPHA
+					};
 
-    switch (Colorspace) {
+
+  switch (Colorspace) {
 
   case PT_GRAY:
            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
@@ -487,11 +563,36 @@ void WriteOutputTags(TIFF *out, int Colorspace, int BytesPerSample)
             TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);    // Needed by TIFF Spec
             break;
 
+        
+
      case PT_HiFi:
 
             TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
             TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 6);
-            TIFFSetField(out, TIFFTAG_INKSET, 2);
+
+            if (StoreAsAlpha) {                                            
+                TIFFSetField(out, TIFFTAG_EXTRASAMPLES, 2, Extra);            
+            }
+            else {            
+                TIFFSetField(out, TIFFTAG_INKSET, 2);
+            }
+
+            TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
+            break;
+
+    case PT_HiFi7:
+
+            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);    
+            TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 7);
+            
+            if (StoreAsAlpha) {                                        
+                TIFFSetField(out, TIFFTAG_EXTRASAMPLES, 3, Extra);            
+            }
+            else {
+            
+                TIFFSetField(out, TIFFTAG_INKSET, 2);
+            }
+
             TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
             break;
 
@@ -499,7 +600,121 @@ void WriteOutputTags(TIFF *out, int Colorspace, int BytesPerSample)
 
             TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
             TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 8);
-            TIFFSetField(out, TIFFTAG_INKSET, 2);
+
+            if (StoreAsAlpha) {                                        
+                TIFFSetField(out, TIFFTAG_EXTRASAMPLES, 4, Extra);            
+            }
+            else {
+            
+                TIFFSetField(out, TIFFTAG_INKSET, 2);
+            }            
+            TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
+            break;
+
+    case PT_HiFi9:
+
+            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
+            TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 9);
+
+            if (StoreAsAlpha) {                                        
+                TIFFSetField(out, TIFFTAG_EXTRASAMPLES, 5, Extra);            
+            }
+            else {
+            
+                TIFFSetField(out, TIFFTAG_INKSET, 2);
+            }            
+            TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
+            break;
+
+    case PT_HiFi10:
+
+            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
+            TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 10);
+
+            if (StoreAsAlpha) {                                        
+                TIFFSetField(out, TIFFTAG_EXTRASAMPLES, 6, Extra);            
+            }
+            else {
+            
+                TIFFSetField(out, TIFFTAG_INKSET, 2);
+            }            
+            TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
+            break;
+
+
+    case PT_HiFi11:
+
+            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
+            TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 11);
+
+            if (StoreAsAlpha) {                                        
+                TIFFSetField(out, TIFFTAG_EXTRASAMPLES, 7, Extra);            
+            }
+            else {
+            
+                TIFFSetField(out, TIFFTAG_INKSET, 2);
+            }            
+            TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
+            break;
+
+
+	 case PT_HiFi12:
+
+            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
+            TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 12);
+
+            if (StoreAsAlpha) {                                        
+                TIFFSetField(out, TIFFTAG_EXTRASAMPLES, 8, Extra);            
+            }
+            else {
+            
+                TIFFSetField(out, TIFFTAG_INKSET, 2);
+            }            
+            TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
+            break;
+
+     case PT_HiFi13:
+
+            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
+            TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 13);
+
+            if (StoreAsAlpha) {                                        
+                TIFFSetField(out, TIFFTAG_EXTRASAMPLES, 9, Extra);            
+            }
+            else {
+            
+                TIFFSetField(out, TIFFTAG_INKSET, 2);
+            }            
+            TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
+            break;
+
+    case PT_HiFi14:
+
+            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
+            TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 14);
+
+            if (StoreAsAlpha) {                                        
+                TIFFSetField(out, TIFFTAG_EXTRASAMPLES, 10, Extra);            
+            }
+            else {
+            
+                TIFFSetField(out, TIFFTAG_INKSET, 2);
+            }            
+            TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
+            break;
+
+    case PT_HiFi15:
+
+            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
+            TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 15);
+
+            if (StoreAsAlpha) {                                        
+                TIFFSetField(out, TIFFTAG_EXTRASAMPLES, 11, Extra);            
+            }
+            else {
+            
+                TIFFSetField(out, TIFFTAG_INKSET, 2);
+            }            
             TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
             break;
 
@@ -596,7 +811,7 @@ void DoEmbedProfile(TIFF* Out, const char* ProfileFile)
         if (f == NULL) return;
 
         size = xfilelength(fileno(f));
-        EmbedBuffer = (LPBYTE) malloc(size + 1);
+        EmbedBuffer = (LPBYTE) _cmsMalloc(size + 1);
         EmbedLen = fread(EmbedBuffer, 1, size, f);
         fclose(f);
         EmbedBuffer[EmbedLen] = 0;
@@ -607,47 +822,156 @@ void DoEmbedProfile(TIFF* Out, const char* ProfileFile)
 
 
 static
-cmsHPROFILE OpenProfile(const char* File)
+void SaveMemoryBlock(const LPBYTE Buffer, DWORD dwLen, const char* Filename)
 {
-       if (!File) 
-            return cmsCreate_sRGBProfile();    
-       
-       if (stricmp(File, "*Lab") == 0)
-                return cmsCreateLabProfile(NULL);
-       
-       if (stricmp(File, "*XYZ") == 0)
-                return cmsCreateXYZProfile();
-         
-       if (stricmp(File, "*Gray22") == 0) {
-           LPGAMMATABLE Gamma = cmsBuildGamma(256, 2.2);
-           cmsHPROFILE hProfile = cmsCreateGrayProfile(cmsD50_xyY(), Gamma);
-           cmsFreeGamma(Gamma);
-           return hProfile;
+    FILE* out = fopen(Filename, "wb");
+    if (out == NULL)
+           FatalError("Cannot create '%s'", Filename);
 
-       }
+    if (fwrite(Buffer, 1, dwLen, out) != dwLen)
+            FatalError("Cannot write %ld bytes to %s", dwLen, Filename);
 
-        return cmsOpenProfileFromFile(File, "r");
+    if (fclose(out) != 0) 
+            FatalError("Error flushing file '%s'", Filename);
 }
 
 
 
+static
+cmsHPROFILE GetTIFFProfile(TIFF* in)
+{    
+    cmsCIExyYTRIPLE Primaries;
+	float* chr;
+    cmsCIExyY WhitePoint;
+    float* wp;
+    int i;       
+    LPGAMMATABLE Gamma[3]; 
+    LPWORD gmr, gmg, gmb;
+    cmsHPROFILE hProfile;
+    DWORD EmbedLen;
+    LPBYTE EmbedBuffer;
+      
+              
+       if (IgnoreEmbedded) return NULL;
+
+       if (TIFFGetField(in, TIFFTAG_ICCPROFILE, &EmbedLen, &EmbedBuffer)) {
+
+              hProfile = cmsOpenProfileFromMem(EmbedBuffer, EmbedLen);
+   
+              if (Verbose) {
+
+                  fprintf(stdout, " (Embedded profile found)\n");
+                  fprintf(stdout, "Product name: %s\n", cmsTakeProductName(hProfile));
+                  fprintf(stdout, "Description : %s\n", cmsTakeProductDesc(hProfile));                          
+                  fflush(stdout);
+              }
+
+              if (hProfile != NULL && SaveEmbedded != NULL)
+                  SaveMemoryBlock(EmbedBuffer, EmbedLen, SaveEmbedded);
+
+              if (hProfile) return hProfile;
+       }
+
+        // Try to see if "colorimetric" tiff
+
+       if (TIFFGetField(in, TIFFTAG_PRIMARYCHROMATICITIES, &chr)) {
+                      
+           Primaries.Red.x   =  chr[0];
+           Primaries.Red.y   =  chr[1];
+           Primaries.Green.x =  chr[2];
+           Primaries.Green.y =  chr[3];
+           Primaries.Blue.x  =  chr[4];
+           Primaries.Blue.y  =  chr[5];
+           
+           Primaries.Red.Y = Primaries.Green.Y = Primaries.Blue.Y = 1.0;
+                      
+           if (TIFFGetField(in, TIFFTAG_WHITEPOINT, &wp)) {
+               
+               WhitePoint.x = wp[0];
+               WhitePoint.y = wp[1];
+               WhitePoint.Y = 1.0;
+                                             
+               // Transferfunction is a bit harder....
+               
+               for (i=0; i < 3; i++)
+                   Gamma[i] = cmsAllocGamma(256);
+                                            
+               TIFFGetFieldDefaulted(in, TIFFTAG_TRANSFERFUNCTION,
+                   &gmr, 
+                   &gmg,
+                   &gmb);
+               
+               CopyMemory(Gamma[0]->GammaTable, gmr, 256*sizeof(WORD));
+               CopyMemory(Gamma[1]->GammaTable, gmg, 256*sizeof(WORD));
+               CopyMemory(Gamma[2]->GammaTable, gmb, 256*sizeof(WORD));
+               
+               hProfile = cmsCreateRGBProfile(&WhitePoint, &Primaries, Gamma);
+               
+               for (i=0; i < 3; i++)
+                   cmsFreeGamma(Gamma[i]);
+
+                if (Verbose) {
+                  fprintf(stdout, " (Colorimetric TIFF)\n");
+                }
+             
+               
+               return hProfile;
+           }
+       }
+
+       return NULL;
+}
+
+
+// Formatter for 8bit Lab TIFF (photometric 8)
+
+static
+unsigned char* UnrollTIFFLab8(register void* nfo, register WORD wIn[], register LPBYTE accum)
+{
+    _LPcmsTRANSFORM info = (_LPcmsTRANSFORM) nfo;
+
+	   wIn[0] = (accum[0]) << 8;
+	   wIn[1] = ((accum[1] > 127) ? (accum[1] - 128) : (accum[1] + 128)) << 8;
+	   wIn[2] = ((accum[2] > 127) ? (accum[2] - 128) : (accum[2] + 128)) << 8;
+       
+       return accum + 3;
+}
+
+
+static
+unsigned char* PackTIFFLab8(register void* nfo, register WORD wOut[], register LPBYTE output)
+{
+    _LPcmsTRANSFORM info = (_LPcmsTRANSFORM) nfo;
+
+		*output++ = (wOut[0] + 0x0080) >> 8;
+
+		wOut[1] = (wOut[1] + 0x0080) >> 8;
+		wOut[2] = (wOut[2] + 0x0080) >> 8;
+
+		*output++ = (wOut[1] < 128) ? (wOut[1] + 128) : (wOut[1] - 128);
+		*output++ = (wOut[2] < 128) ? (wOut[2] + 128) : (wOut[2] - 128);
+
+       return output;
+}
 
 
 // Transform one image
 
 static
-int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
+int TransformImage(TIFF* in, TIFF* out, const char *cDefInpProf, const char *cOutProf)
 {
        cmsHPROFILE hIn, hOut, hProof, hInkLimit = NULL;
        cmsHTRANSFORM xform;
        DWORD wInput, wOutput;
        int OutputColorSpace;
        int bps = (Width16 ? 2 : 1);
-       DWORD dwFlags = 0; 
-       DWORD EmbedLen;
-       LPBYTE EmbedBuffer;
+       DWORD dwFlags = 0;        
        int nPlanes;
 
+    // Observer adaptation state (only meaningful on absolute colorimetric intent)
+
+       cmsSetAdaptationState(ObserverAdaptationState);
+
        if (EmbedProfile && cOutProf) 
            DoEmbedProfile(out, cOutProf);
 
@@ -656,7 +980,12 @@ int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
        if (BlackWhiteCompensation) 
             dwFlags |= cmsFLAGS_WHITEBLACKCOMPENSATION;           
        
-       
+
+       if (PreserveBlack) {
+			dwFlags |= cmsFLAGS_PRESERVEBLACK;
+			if (PrecalcMode == 0) PrecalcMode = 1;
+            cmsSetCMYKPreservationStrategy(PreserveBlack-1);
+	   }
 
        switch (PrecalcMode) {
            
@@ -682,29 +1011,16 @@ int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
        }
        else {
 
-                if (!IgnoreEmbedded && 
-                    TIFFGetField(in, TIFFTAG_ICCPROFILE, &EmbedLen, &EmbedBuffer)) {
-
-                      hIn = cmsOpenProfileFromMem(EmbedBuffer, EmbedLen);
-                      if (Verbose) {
-                          fprintf(stdout, " (Embedded profile found)\n");
-                          fprintf(stdout, "Product name: %s\n", cmsTakeProductName(hIn));
-                          fprintf(stdout, "Description : %s\n", cmsTakeProductDesc(hIn));                          
-                          fflush(stdout);
-                      }
-
-                }
-                else
-                {                    
-                       hIn = OpenProfile(cDefInpProf);
-                }
+               hIn =  GetTIFFProfile(in);
 
+               if (hIn == NULL)                    
+                       hIn = OpenStockProfile(cDefInpProf);               
                
-                 hOut = OpenProfile(cOutProf);
+               hOut = OpenStockProfile(cOutProf);
                  
-                 if (cProofing != NULL) {
+               if (cProofing != NULL) {
 
-                   hProof = cmsOpenProfileFromFile(cProofing, "r");
+                   hProof = OpenStockProfile(cProofing);
                    dwFlags |= cmsFLAGS_SOFTPROOFING;
                }
        }
@@ -715,14 +1031,14 @@ int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
 
        // Assure both, input profile and input TIFF are on same colorspace
 
-       if (cmsGetColorSpace(hIn) != _cmsICCcolorSpace(T_COLORSPACE(wInput)))
+       if (_cmsLCMScolorSpace(cmsGetColorSpace(hIn)) != (int) T_COLORSPACE(wInput))
               FatalError("Input profile is not operating in proper color space");
 
       
        if (!lIsDeviceLink) 
-                OutputColorSpace = ICC2LCMS(cmsGetColorSpace(hOut));
+                OutputColorSpace = _cmsLCMScolorSpace(cmsGetColorSpace(hOut));
        else 
-                OutputColorSpace = ICC2LCMS(cmsGetPCS(hIn));
+                OutputColorSpace = _cmsLCMScolorSpace(cmsGetPCS(hIn));
                 
        wOutput      = ComputeOutputFormatDescriptor(wInput, OutputColorSpace, bps);
 
@@ -754,13 +1070,14 @@ int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
        }
        else {
 
-       xform = cmsCreateProofingTransform(hIn, wInput, 
-                                          hOut, wOutput, 
-                                          hProof, Intent, 
-                                          ProofingIntent, 
-                                          dwFlags);
+		   xform = cmsCreateProofingTransform(hIn, wInput, 
+											  hOut, wOutput, 
+											  hProof, Intent, 
+											  ProofingIntent, 
+											  dwFlags);
        }
 
+      
 
        // Planar stuff
 
@@ -769,6 +1086,26 @@ int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
        else
             nPlanes = 1;
 
+
+	   // TIFF Lab of 8 bits need special handling
+
+		if (wInput == TYPE_Lab_8 && 
+			   !InputLabUsingICC &&
+			   cInpProf != NULL  &&
+			   stricmp(cInpProf, "*Lab") == 0) {
+
+					cmsSetUserFormatters(xform, TYPE_Lab_8, UnrollTIFFLab8, TYPE_Lab_8, NULL); 
+		}
+
+
+		if (wOutput == TYPE_Lab_8 && 			   
+			   cOutProf != NULL  &&
+			   stricmp(cOutProf, "*Lab") == 0) {
+
+					cmsSetUserFormatters(xform, TYPE_Lab_8, NULL, TYPE_Lab_8, PackTIFFLab8); 
+		}
+
+	   
        // Handle tile by tile or strip by strip
 
        if (TIFFIsTiled(in)) {
@@ -801,7 +1138,7 @@ int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
 static
 void Help(int level)
 {
-    fprintf(stderr, "little cms ICC profile applier for TIFF - v3.3\n\n");
+    fprintf(stderr, "little cms ICC profile applier for TIFF - v5.0\n\n");
     fflush(stderr);
     
      switch(level) {
@@ -818,21 +1155,27 @@ void Help(int level)
      fprintf(stderr, "%cl<profile> - Transform by device-link profile\n", SW);   
      fprintf(stderr, "%ct<0,1,2,3> - Intent (0=Perceptual, 1=Colorimetric, 2=Saturation, 3=Absolute)\n", SW);    
      fprintf(stderr, "\n");
-
-     fprintf(stderr, "%cw - Wide output (generates 16 bps tiff)\n", SW);
-    
-     fprintf(stderr, "%cn - Ignore embedded profile on input\n", SW);
-     fprintf(stderr, "%ce - Embed destination profile\n", SW);
      fprintf(stderr, "%cc<0,1,2,3> - Precalculates transform (0=Off, 1=Normal, 2=Hi-res, 3=LoRes) [defaults to 1]\n", SW);     
      fprintf(stderr, "\n");
 
+     fprintf(stderr, "%cw - Wide output (generates 16 bps tiff)\n", SW);
+     fprintf(stderr, "%ca - Handle channels > 4 as alpha\n", SW);
+
+     fprintf(stderr, "%cn - Ignore embedded profile on input\n", SW);
+     fprintf(stderr, "%ce - Embed destination profile\n", SW);
+     fprintf(stderr, "%cs<new profile> - Save embedded profile as <new profile>\n", SW);
+     fprintf(stderr, "\n");
+
+     
      fprintf(stderr, "%cp<profile> - Soft proof profile\n", SW);
      fprintf(stderr, "%cm<0,1,2,3> - Soft proof intent\n", SW);
      fprintf(stderr, "%cg - Marks out-of-gamut colors on softproof\n", SW);
      
      fprintf(stderr, "\n"); 
      fprintf(stderr, "%cb - Black point compensation\n", SW);
+	 fprintf(stderr, "%cf<n> - Preserve black (CMYK only) 0=off, 1=black ink only, 2=full K plane\n", SW);
      fprintf(stderr, "%ck<0..400> - Ink-limiting in %% (CMYK only)\n", SW);
+     fprintf(stderr, "%cd<0..1> - Observer adaptation state (abs.col. only)\n", SW);
 
      fprintf(stderr, "\n");
      fprintf(stderr, "%ch<0,1,2> - More help\n", SW);
@@ -855,7 +1198,7 @@ void Help(int level)
                      "To recover sRGB from a CMYK separation:\n"
                      "\ttifficc %ciprinter.icm incmyk.tif outrgb.tif\n"
                      "To convert from CIELab TIFF to sRGB\n"
-                     "\ttifficc %ciTiffLab8Spac.icm in.tif out.tif\n\n", 
+                     "\ttifficc %ci*Lab in.tif out.tif\n\n", 
                      SW, SW, SW, SW, SW, SW);
      break;
 
@@ -883,19 +1226,49 @@ void HandleSwitches(int argc, char *argv[])
 {
        int s;
       
-       while ((s=xgetopt(argc,argv,"eEbBwWnNvVGgh:H:i:I:o:O:P:p:t:T:c:C:l:L:M:m:K:k:")) != EOF) {
+       while ((s=xgetopt(argc,argv,"aAeEbBwWnNvVGgh:H:i:I:o:O:P:p:t:T:c:C:l:L:M:m:K:k:S:s:D:d:f:F:")) != EOF) {
 
        switch (s)
        {
 
+       case 'a':
+       case 'A':
+            StoreAsAlpha = TRUE;
+            break;
+       case 'b':
+       case 'B':
+            BlackWhiteCompensation = TRUE;
+            break;
+       
+       case 'c':
+       case 'C':
+            PrecalcMode = atoi(xoptarg);
+            if (PrecalcMode < 0 || PrecalcMode > 3)
+                    FatalError("Unknown precalc mode '%d'", PrecalcMode);
+            break;
+
+       case 'd':
+       case 'D': ObserverAdaptationState = atof(xoptarg);
+                 if (ObserverAdaptationState != 0 && 
+                     ObserverAdaptationState != 1.0)
+                        Warning("Adaptation states other that 0 or 1 are not yet implemented");
+                 break;
+
        case 'e':
        case 'E':
             EmbedProfile = TRUE;
             break;
 
-       case 'b':
-       case 'B':
-            BlackWhiteCompensation = TRUE;
+	   case 'f':
+	   case 'F':
+		    PreserveBlack = atoi(xoptarg);
+            if (PreserveBlack < 0 || PreserveBlack > 2)
+                    FatalError("Unknown PreserveBlack '%d'", PreserveBlack);
+			break;
+
+        case 'g':
+        case 'G':
+            GamutCheck = TRUE;
             break;
 
        case 'v':
@@ -955,18 +1328,7 @@ void HandleSwitches(int argc, char *argv[])
             Width16 = TRUE;
             break;
 
-        case 'g':
-        case 'G':
-            GamutCheck = TRUE;
-            break;
-
-        case 'c':
-        case 'C':
-            PrecalcMode = atoi(xoptarg);
-            if (PrecalcMode < 0 || PrecalcMode > 3)
-                    FatalError("Unknown precalc mode '%d'", PrecalcMode);
-            break;
-
+           
         case 'k':
         case 'K':
                 InkLimit = atof(xoptarg);
@@ -975,6 +1337,10 @@ void HandleSwitches(int argc, char *argv[])
                 break;
                 
 
+        case 's':
+        case 'S': SaveEmbedded = xoptarg;
+                  break;
+                  
         case 'H':
         case 'h':  {
 
@@ -1003,6 +1369,8 @@ int main(int argc, char* argv[])
                          "absolute colorimetric" };
 
       HandleSwitches(argc, argv);
+  
+	  cmsSetErrorHandler(MyErrorHandler);
 
       if ((argc - xoptind) != 2) {
 
